# -*- coding: utf-8 -*-
import unittest

from wikipedia import wikipedia
from request_mock_data import mock_data


# mock out _wiki_request
def _wiki_request(**params):
    return mock_data["_wiki_request calls"][tuple(sorted(params.items()))]
wikipedia._wiki_request = _wiki_request


class TestPageSetUp(unittest.TestCase):
    """Test the functionality of wikipedia.page's __init__ and load functions."""

    def test_missing(self):
        """Test that page raises a PageError for a nonexistant page."""
        # Callicarpa?
        purpleberry = lambda: wikipedia.page("purpleberry", auto_suggest=False)
        self.assertRaises(wikipedia.PageError, purpleberry)

    def test_redirect_true(self):
        """Test that a page successfully redirects a query."""
        # no error should be raised if redirect is test_redirect_true
        mp = wikipedia.page("Menlo Park, New Jersey")

        self.assertEqual(mp.title, "Edison, New Jersey")
        self.assertEqual(mp.url, "http://en.wikipedia.org/wiki/Edison,_New_Jersey")

    def test_redirect_false(self):
        """Test that page raises an error on a redirect when redirect == False."""
        mp = lambda: wikipedia.page("Menlo Park, New Jersey", auto_suggest=False, redirect=False)
        self.assertRaises(wikipedia.RedirectError, mp)

    def test_disambiguate(self):
        """Test that page raises an error when a disambiguation page is reached."""
        try:
            ram = wikipedia.page("Dodge Ram (disambiguation)", auto_suggest=False, redirect=False)
            error_raised = False
        except wikipedia.DisambiguationError as e:
            error_raised = True
            options = e.options

        self.assertTrue(error_raised)
        self.assertEqual(options, [u'Dodge Ramcharger', u'Dodge Ram Van', u'Dodge Mini Ram', u'Dodge Caravan C/V', u'Dodge Caravan C/V', u'Ram C/V', u'Dodge Ram 50', u'Dodge D-Series', u'Dodge Rampage', u'Ram (brand)'])

    def test_auto_suggest(self):
        """Test that auto_suggest properly corrects a typo."""
        # yum, butter.
        butterfly = wikipedia.page("butteryfly")

        self.assertEqual(butterfly.title, "butterfly")
        self.assertEqual(butterfly.url, "http://en.wikipedia.org/wiki/Butterfly")


class TestPage(unittest.TestCase):
    """Test the functionality of the rest of wikipedia.page."""

    def setUp(self):
        # one of the shortest wikipedia articles that includes images
        self.celtuce = wikipedia.page("Celtuce")

    def test_title(self):
        """Test the title."""
        self.assertEqual(self.celtuce.title, "Celtuce")

    def test_url(self):
        """Test the url."""
        self.assertEqual(self.celtuce.url, "http://en.wikipedia.org/wiki/Celtuce")

    def test_content(self):
        """Test the plain text content."""
        self.assertEqual(self.celtuce.content, mock_data['data']["celtuce.content"])

    def test_summary(self):
        """Test the summary."""
        self.assertEqual(self.celtuce.summary, mock_data['data']["celtuce.summary"])

    def test_images(self):
        """Test the list of image URLs."""
        self.assertEqual(sorted(self.celtuce.images), mock_data['data']["celtuce.images"])

    def test_references(self):
        """Test the list of reference URLs."""
        self.assertEqual(self.celtuce.references, mock_data['data']["celtuce.references"])

    def test_links(self):
        """Test the list of titles of links to Wikipedia pages."""
        self.assertEqual(self.celtuce.links, mock_data['data']["celtuce.links"])

    def test_html(self):
        """Test the full HTML method."""
        self.assertEqual(self.celtuce.html(), mock_data['data']["celtuce.html"])



import requests
from bs4 import BeautifulSoup

from .exceptions import *
from .util import cache, stdout_encode

API_URL = 'http://en.wikipedia.org/w/api.php'

def set_lang(prefix):
    '''
    Change the language of the API being requested. 
    Set `prefix` to one of the two letter prefixes found on the `list of all Wikipedias <http://meta.wikimedia.org/wiki/List_of_Wikipedias>`_.

    After setting the language, the cache for ``search``, ``suggest``, and ``summary`` will be cleared.

    .. note:: Make sure you search for page titles in the language that you have set.
    '''
    global API_URL
    API_URL = 'http://' + prefix.lower() + '.wikipedia.org/w/api.php'

    for cached_func in (search, suggest, summary):
        cached_func.clear_cache()


@cache
def search(query, results=10, suggestion=False):
    '''
    Do a Wikipedia search for `query`.

    Keyword arguments:

    * results - the maxmimum number of results returned
    * suggestion - if True, return results and suggestion (if any) in a tuple
    '''

    search_params = {
        'list': 'search',
        'srprop': '',
        'srlimit': results
    }
    if suggestion:
        search_params['srinfo'] = 'suggestion'
    search_params['srsearch'] = query
    search_params['limit'] = results

    raw_results = _wiki_request(**search_params)
    search_results = (d['title'] for d in raw_results['query']['search'])

    if suggestion:
        if raw_results['query'].get('searchinfo'):
            return list(search_results), raw_results['query']['searchinfo']['suggestion']
        else:
            return list(search_results), None

    return list(search_results)


@cache
def suggest(query):
    '''
    Get a Wikipedia search suggestion for `query`.
    Returns a string or None if no suggestion was found.
    '''

    search_params = {
        'list': 'search',
        'srinfo': 'suggestion',
        'srprop': '',
    }
    search_params['srsearch'] = query

    raw_result = _wiki_request(**search_params)

    if raw_result['query'].get('searchinfo'):
        return raw_result['query']['searchinfo']['suggestion']

    return None


def random(pages=1):
    '''
    Get a list of random Wikipedia article titles.

    .. note:: Random only gets articles from namespace 0, meaning no Category, User talk, or other meta-Wikipedia pages.

    Keyword arguments:

    * pages - the number of random pages returned (max of 10)
    '''
    #http://en.wikipedia.org/w/api.php?action=query&list=random&rnlimit=5000&format=jsonfm
    query_params = {
        'list': 'random',
        'rnnamespace': 0,
        'rnlimit': pages,
    }

    request = _wiki_request(**query_params)
    titles = [page['title'] for page in request['query']['random']]

    if len(titles) == 1:
        return titles[0]

    return titles


@cache
def summary(title, sentences=0, chars=0, auto_suggest=True, redirect=True):
    '''
    Plain text summary of the page.

    .. note:: This is a convenience wrapper - auto_suggest and redirect are enabled by default

    Keyword arguments:

    * sentences - if set, return the first `sentences` sentences
    * chars - if set, return only the first `chars` characters.
    * auto_suggest - let Wikipedia find a valid page title for the query
    * redirect - allow redirection without raising RedirectError
    '''

    # use auto_suggest and redirect to get the correct article
    # also, use page's error checking to raise DisambiguationError if necessary
    page_info = page(title, auto_suggest=auto_suggest, redirect=redirect)
    title = page_info.title
    pageid = page_info.pageid

    query_params = {
        'prop': 'extracts',
        'explaintext': '',
        'titles': title
    }

    if sentences:
        query_params['exsentences'] = sentences
    elif chars:
        query_params['exchars'] = chars
    else:
        query_params['exintro'] = ''

    request = _wiki_request(**query_params)
    summary = request['query']['pages'][pageid]['extract']

    return summary


def page(title, auto_suggest=True, redirect=True, preload=False):
    '''
    Get a WikipediaPage object for the page with title `title`.

    Keyword arguments:

    * auto_suggest - let Wikipedia find a valid page title for the query
    * redirect - allow redirection without raising RedirectError
    * preload - load content, summary, images, references, and links during initialization
    '''

    if auto_suggest:
        results, suggestion = search(title, results=1, suggestion=True)
        try:
            title = suggestion or results[0]
        except IndexError:
            # if there is no suggestion or search results, the page doesn't exist
            raise PageError(title)

    return WikipediaPage(title, redirect=redirect, preload=preload)


class WikipediaPage(object):
    '''
    Contains data from a Wikipedia page.
    Uses property methods to filter data from the raw HTML.
    '''

    def __init__(self, title, redirect=True, preload=False, original_title=''):
        self.title = title
        self.original_title = original_title or title

        self.load(redirect=redirect, preload=preload)

        if preload:
            for prop in ['content', 'summary', 'images', 'references', 'links']:
                getattr(self, prop)

    def __repr__(self):
        return stdout_encode(u'<WikipediaPage \'{}\'>'.format(self.title))

    def load(self, redirect=True, preload=False):
        '''
        Load basic information from Wikipedia.
        Confirm that page exists and is not a disambiguation/redirect.
        '''

        query_params = {
            'prop': 'info|pageprops',
            'inprop': 'url',
            'ppprop': 'disambiguation',
            'titles': self.title
        }

        request = _wiki_request(**query_params)
        pageid = list(request['query']['pages'].keys())[0]
        data = request['query']['pages'][pageid]

        # missing is equal to empty string if it is True
        if data.get('missing') == '':
            raise PageError(self.title)

        # same thing for redirect
        elif data.get('redirect') == '':
            if redirect:
                # change the title and reload the whole object
                query_params = {
                    'prop': 'extracts',
                    'explaintext': '',
                    'titles': self.title
                }

                request = _wiki_request(**query_params)
                title = request['query']['pages'][pageid]['extract'].split('\n')[0][8:].strip()

                self.__init__(title, redirect=redirect, preload=preload)

            else:
                raise RedirectError(self.title)

        # since we only asked for disambiguation in ppprop,
        # if a pageprop is returned,
        # then the page must be a disambiguation page
        elif data.get('pageprops'):
            request = _wiki_request(titles=self.title, prop='revisions', rvprop='content', rvparse='', rvlimit=1)
            html = request['query']['pages'][pageid]['revisions'][0]['*']

            lis = BeautifulSoup(html).find_all('li')
            filtered_lis = [li for li in lis if not 'tocsection' in ''.join(li.get('class', []))]
            may_refer_to = [li.a.get_text() for li in filtered_lis if li.a]

            raise DisambiguationError(self.title, may_refer_to)

        else:
            self.pageid = pageid
            self.url = data['fullurl']

    def html(self):
        '''
        Get full page HTML.

        .. warning:: this can get pretty slow on long pages.
        '''

        if not getattr(self, '_html', False):
            query_params = {
                'prop': 'revisions',
                'rvprop': 'content',
                'rvlimit': 1,
                'rvparse': '',
                'titles': self.title
            }

            request = _wiki_request(**query_params)
            self._html = request['query']['pages'][self.pageid]['revisions'][0]['*']

        return self._html

    @property
    def content(self):
        '''
        Plain text content of the page, excluding images, tables, and other data.
        '''

        if not getattr(self, '_content', False):
            query_params = {
                'prop': 'extracts',
                'explaintext': '',
                'titles': self.title
            }

            request = _wiki_request(**query_params)
            self._content = request['query']['pages'][self.pageid]['extract']

        return self._content

    @property
    def summary(self):
        '''
        Plain text summary of the page.
        '''

        if not getattr(self, '_summary', False):
            query_params = {
                'prop': 'extracts',
                'explaintext': '',
                'exintro': '',
                'titles': self.title
            }

            request = _wiki_request(**query_params)
            self._summary = request['query']['pages'][self.pageid]['extract']

        return self._summary

    @property
    def images(self):
        '''
        List of URLs of images on the page.
        '''

        if not getattr(self, '_images', False):
            query_params = {
                'generator': 'images',
                'gimlimit': 'max',
                'prop': 'imageinfo',
                'iiprop': 'url',
                'titles': self.title,
            }

            request = _wiki_request(**query_params)

            image_keys = request['query']['pages'].keys()
            images = (request['query']['pages'][key] for key in image_keys)
            self._images = [image['imageinfo'][0]['url'] for image in images if image.get('imageinfo')]

        return self._images

    @property
    def references(self):
        '''
        List of URLs of external links on a page.
        May include external links within page that aren't technically cited anywhere.
        '''

        if not getattr(self, '_references', False):
            query_params = {
                'prop': 'extlinks',
                'ellimit': 'max',
                'titles': self.title,
            }

            request = _wiki_request(**query_params)

            links = request['query']['pages'][self.pageid]['extlinks']
            relative_urls = (link['*'] for link in links)

            def add_protocol(url):
                return url if url.startswith('http') else 'http:' + url

            self._references = [add_protocol(url) for url in relative_urls]

        return self._references

    @property
    def links(self):
        '''
        List of titles of Wikipedia page links on a page.

        .. note:: Only includes articles from namespace 0, meaning no Category, User talk, or other meta-Wikipedia pages.
        '''

        if not getattr(self, '_links', False):
            self._links = []

            request = {
                'prop': 'links',
                'plnamespace': 0,
                'pllimit': 'max',
                'titles': self.title,
            }
            lastContinue = {}

            # based on https://www.mediawiki.org/wiki/API:Query#Continuing_queries
            while True:
                params = request.copy()
                params.update(lastContinue)
                
                request = _wiki_request(**params)
                self._links.extend([link['title'] for link in request['query']['pages'][self.pageid]['links']])

                if 'continue' not in request:
                    break

                lastContinue = request['continue']

        return self._links


def donate():
    '''
    Open up the Wikimedia donate page in your favorite browser.
    '''
    import webbrowser
    webbrowser.open('https://donate.wikimedia.org/w/index.php?title=Special:FundraiserLandingPage', new=2)


def _wiki_request(**params):
    '''
    Make a request to the Wikipedia API using the given search parameters.
    Returns a parsed dict of the JSON response.
    '''
    params['format'] = 'json'
    params['action'] = 'query'

    headers = {
        'User-Agent': 'wikipedia (https://github.com/goldsmith/Wikipedia/)'
    }

    r = requests.get(API_URL, params=params, headers=headers)

    return r.json()



"""
Global wikipedia excpetion and warning classes.
"""

import sys

class WikipediaException(Exception):
    """Base Wikipedia exception class."""

    if sys.version_info > (3, 0):
        def __str__(self):
            return self.__unicode__()

    else:
        def __str__(self):
            return self.__unicode__().encode('utf8')


class PageError(WikipediaException):
    """Exception raised when no Wikipedia matched a query."""

    def __init__(self, page_title):
        self.title = page_title

    def __unicode__(self):
        return u"\"{0}\" does not match any pages. Try another query!".format(self.title)


class DisambiguationError(WikipediaException):
    """
    Exception raised when a page resolves to a Disambiguation page.

    The `options` property contains a list of titles
    of Wikipedia pages that the query may refer to.

    .. note:: `options` does not include titles that do not link to a valid Wikipedia page.
    """

    def __init__(self, title, may_refer_to):
        self.title = title
        self.options = may_refer_to

    def __unicode__(self):
        return u"\"{0}\" may refer to: \n{1}".format(self.title, '\n'.join(self.options))


class RedirectError(WikipediaException):
    """Exception raised when a page title unexpectedly resolves to a redirect."""

    def __init__(self, page_title):
        self.title = page_title

    def __unicode__(self):
        return u"\"{0}\" resulted in a redirect. Set the redirect property to True to allow automatic redirects.".format(self.title)



# -*- coding: utf-8 -*-
import os
import codecs

from setuptools import setup

long_description = codecs.open(os.path.join(os.path.dirname(__file__), 'README.rst'), 'r', 'utf-8').read()

dependencies = [
    "beautifulsoup4",
    "requests==1.2.3"
]

setup(
    name = "wikipedia",
    version = "1.0.3",
    author = "Jonathan Goldsmith",
    author_email = "jhghank@gmail.com",
    description = "Wikipedia API for Python",
    license = "MIT",
    keywords = "python wikipedia API",
    url = "https://github.com/goldsmith/Wikipedia",
    install_requires = dependencies,
    packages = ['wikipedia'],
    long_description = long_description,
    classifiers = (
        'Development Status :: 4 - Beta',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3'
    ),
)
